package genius.core.uncertainty;

import java.util.List;

import java.util.ArrayList;
import java.util.Iterator;

import agents.org.apache.commons.lang.StringUtils;
import genius.core.Bid;

/**
 * Provides a (total) ranking of bids:
 * b1 <= b2 <= ... <= bn
 */
public class BidRanking implements Iterable<Bid>
{
	/** Ordered from low to high */
	List<Bid> bidOrder;
	
	public BidRanking(List<Bid> bidOrder) 
	{
		this.bidOrder = bidOrder;
	}

	public Bid getMinimalBid()
	{
		return bidOrder.get(0);
	}
	
	public Bid getMmaximalBid()
	{
		return bidOrder.get(bidOrder.size() - 1);
	}
	
	public List<OutcomeComparison> getPairwiseComparisons()
	{
		ArrayList<OutcomeComparison> comparisons = new ArrayList<OutcomeComparison>();
		for (int i = 0; i < bidOrder.size() - 1; i++)
			comparisons.add(new OutcomeComparison(bidOrder.get(i), bidOrder.get(i+1), -1));
		return comparisons;
		
	}
	
	public int getClosestBidRank(Bid bid)
	{
		Bid closestBid = null;
		double closestDistance = Double.MAX_VALUE;
		int rank = 0;
		int closestRank = 0;
			
		for (Bid b : bidOrder)
		{
			double d = 1 / (double) b.countEqualValues(bid);
			if (d < closestDistance)
			{
				closestDistance = d;
				closestBid = b;
				closestRank = rank;
			}
			rank++;
		}
		return closestRank;
	}
	
	public List<Bid> getBidOrder() 
	{
		return bidOrder;
	}
	
	/**
	 * The size equals 1 + the number of comparisons
	 * @return
	 */
	public int getSize()
	{
		return bidOrder.size();
	}
	
	public int getAmountOfComparisons()
	{
		return getSize() - 1;
	}
	
	@Override
	public String toString() 
	{
		return StringUtils.join(bidOrder.iterator(), " <= ");
	}

	/**
	 * Iterates the bids from low to high
	 */
	@Override
	public Iterator<Bid> iterator() 
	{
		return bidOrder.iterator();
	}

}
